(****************************************************************************)
(*                                                                          *)
(*  Program language:  Borland Turbo Pascal  Version 6.0                    *)
(*  Program title: JDay2                                                    *)
(*                                                                          *)
(*  Written by T. R. Willmitch     1992 January 23                          *)
(*                                                                          *)
(*  Based on the Julian Day number algorithm described on pages 60-62 of    *)
(*  ASTRONOMICAL ALGORITHMS by Jean Meeus (Richmond, 1991).                 *)
(*                                                                          *)
(*  Program summary:  Given the Julian or Gregorian calendar date and the   *)
(*                    Universal Time, this program determines the Julian    *)
(*                    Day number (JD) and day fraction.                     *)
(*                                                                          *)
(*                    N.B., All dates beginning with 1582 October 15 are    *)
(*                    assumed to be Gregorian, while earlier dates are      *)
(*                    assumed to be Julian.                                 *)
(*                                                                          *)
(****************************************************************************)

PROGRAM JDay2;

USES Crt;

VAR  Year, Month, Day, Hour, Minute : INTEGER;
      DecimalDay, Second, JulianDay : DOUBLE;
                           Response : STRING;


PROCEDURE InputData;

VAR  Input : BOOLEAN;

BEGIN
  REPEAT
    Writeln('Enter the date and Universal Time:':39);
    Writeln;
    Writeln;
    Write('Date (YYYY MM DD)? ':29);
    Readln(Year, Month, Day);
    Writeln;
    Write('Time (HH MM SS.S)? ':29);
    Readln(Hour, Minute, Second);
    Writeln; Writeln;

    Input := TRUE;

(* Confirm that the date is within the proper range.                        *)

    IF (Month < 1) OR (Month > 12) THEN Input := FALSE;
    CASE Month OF
         1 : IF (Day < 1) OR (Day > 31) THEN Input := FALSE;
         2 : IF (Day < 1) OR (Day > 29) THEN Input := FALSE;
         3 : IF (Day < 1) OR (Day > 31) THEN Input := FALSE;
         4 : IF (Day < 1) OR (Day > 30) THEN Input := FALSE;
         5 : IF (Day < 1) OR (Day > 31) THEN Input := FALSE;
         6 : IF (Day < 1) OR (Day > 30) THEN Input := FALSE;
      7..8 : IF (Day < 1) OR (Day > 31) THEN Input := FALSE;
         9 : IF (Day < 1) OR (Day > 30) THEN Input := FALSE;
        10 : IF (Day < 1) OR (Day > 31) THEN Input := FALSE;
        11 : IF (Day < 1) OR (Day > 30) THEN Input := FALSE;
        12 : IF (Day < 1) OR (Day > 31) THEN Input := FALSE;
     END;  (* CASE *)

(* Confirm that the time is within the proper range.                        *)

    IF (Hour < 0) OR (Hour > 23) THEN Input := FALSE;
    IF (Minute < 0) OR (Minute > 59) THEN Input := FALSE;
    IF (Second < 0) OR (Second >= 60) THEN Input := FALSE;

  UNTIL Input = TRUE
END;  (* InputData *)


FUNCTION FracDay (Hour, Minute : INTEGER;
                        Second : DOUBLE) : DOUBLE;

(* The function FracDay converts the time (in hours, minutes, and seconds)  *)
(* into a fractional day.                                                   *)

BEGIN
  FracDay := Hour / 24 + Minute / 1440 + Second / 86400
END;  (* FUNCTION FracDay *)


FUNCTION JulDay (Year, Month : INTEGER; Day : DOUBLE) : DOUBLE;

(* The function JulDay converts the Julian or Gregorian calendar date into  *)
(* its coresponding Julian Day number.  The day may be given as a decimal   *)
(* day, using Universal Time.                                               *)
(*                                                                          *)
(* N.B.,  All dates beginning with 1582 October 15 are assumed to be        *)
(* Gregorian, while earlier dates are assumed to be Julian.                 *)


VAR       A, B : LONGINT;
     Gregorian : BOOLEAN;

BEGIN
  Gregorian := TRUE;

(* Check if the date is Gregorian, beginning 1582 October 15.               *)

  IF Year < 1582 THEN Gregorian := FALSE;
  IF (Year = 1582) AND (Month < 10) THEN Gregorian := FALSE;
  IF (Year = 1582) AND (Month = 10) THEN
    IF Day < 15 THEN Gregorian := FALSE;

  IF Month < 3 THEN BEGIN
    Year := Year - 1;
    Month := Month + 12
  END;  (* IF *)

  A := Year DIV 100;

  IF Gregorian THEN
    B := 2 - A + A DIV 4
  ELSE
    B := 0;

  JulDay := INT(365.25 * (Year + 4716)) + INT(30.6001 * (Month +1)) + Day + B
          - 1524.5
END;  (* FUNCTION JulDay *)


BEGIN   (* Main program *)
  REPEAT
    ClrScr;
    GotoXY(31,5);
    Writeln('Julian Day Number');
    Writeln;
    Writeln;
    InputData;
    DecimalDay := Day + FracDay(Hour, Minute, Second);
    JulianDay := JulDay(Year, Month, DecimalDay);
    Writeln('JD = ':35, JulianDay:14:6);
    Writeln;
    Writeln;
    Write('Determine another JD (y/n)? ':54);
    Readln(Response);
  UNTIL (Response = 'n') OR (Response = 'N')
END.  (* Main program *)
